<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
require_once('wc-payment-class.php');
class Woo_Ipara_Gateway extends WC_Payment_Gateway{
    public function __construct(){
        $this->id = 'wc_ipara';
        $this->method_title = __('IPARA', WIG_TEXT_DOMAIN);
        $this->method_description = __('Woocommerce payment gateway for IPARA', WIG_TEXT_DOMAIN);
        $this->title =$this->get_option('title');
        $this->description =$this->get_option('description')?$this->get_option('description'): __('Credit/Deit Card', WIG_TEXT_DOMAIN);
        $this->has_fields = true;
        $this->init_form_fields();
        $this->init_settings();
        $this->mode=$this->get_option('3dMode')?$this->get_option('3dMode'):'off';
        $this->installment=$this->get_option('installment')?$this->get_option('installment'):'no';
        $this->supports[] = 'default_credit_card_form';
        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, [&$this, 'process_admin_options']);
        add_action('woocommerce_credit_card_form_start',[&$this,'add_installment_data'],10,1);
    }
    
    public function init_form_fields(){
        $this->form_fields=[
            'enabled' => [
                'title' => __( 'Enable/Disable', WIG_TEXT_DOMAIN ),
                'type' => 'checkbox',
                'label' => __( 'Enable IPARA', WIG_TEXT_DOMAIN ),
                'default' => 'yes'
            ],
            'title' => [
                'title' => __( 'Title', 'woocommerce' ),
                'type' => 'text',
                'description' => __( 'This controls the title which the user sees during checkout.', WIG_TEXT_DOMAIN ),
                'default' => __( 'IPARA Payment', WIG_TEXT_DOMAIN ),
                'desc_tip'      => true,
            ],
            'description' => [
                'title' => __( 'Description', WIG_TEXT_DOMAIN ),
                'type' => 'textarea',
                'description' => __( 'This controls the description which the user sees during checkout.', WIG_TEXT_DOMAIN ),
                'default' => __( 'Pay with Credit/Debit card', WIG_TEXT_DOMAIN )
            ],
            'publicKey' => [
                'title' => __( 'Public Key', WIG_TEXT_DOMAIN ),
                'type' => 'text',
                'description' => '',
                'default' => '',
            ],
            'privateKey' => [
                'title' => __( 'Private Key', WIG_TEXT_DOMAIN ),
                'type' => 'text',
                'description' => '',
                'default' => '',
            ],
            '3dMode' => [
                'title' => __( '3D Mode', WIG_TEXT_DOMAIN ),
                'type' => 'select',
                'description' => '',
                'default' => '',
                'options' => [
                    'on' => __('On',WIG_TEXT_DOMAIN),
                    'auto' => __('Auto',WIG_TEXT_DOMAIN),
                    'off' => __('Off',WIG_TEXT_DOMAIN),
                ],
            ],
            'installment' => [
                'title' => __( 'Installment', WIG_TEXT_DOMAIN ),
                'type' => 'checkbox',
                'label' => __( 'Installment Payment Support', WIG_TEXT_DOMAIN ),
                'default' => 'no'
            ],
            'shipping' => [
                'title' => __( 'Shipping Charge', WIG_TEXT_DOMAIN ),
                'type' => 'checkbox',
                'label' => __( 'Include Shipping Charge', WIG_TEXT_DOMAIN ),
                'default' => 'no'
            ],
            'tax' => [
                'title' => __( 'Total Tax', WIG_TEXT_DOMAIN ),
                'type' => 'checkbox',
                'label' => __( 'Include Tax', WIG_TEXT_DOMAIN ),
                'default' => 'no'
            ],
             'testMode' => [
                'title' => __( 'Sandbox/Production', WIG_TEXT_DOMAIN ),
                'type' => 'checkbox',
                'label' => __( 'Use IPARA Sandbox', WIG_TEXT_DOMAIN ),
                'default' => 'no'
             ]
        ];
    }
    public function add_installment_data($gatewayId){
        if($gatewayId==$this->id){        
            if($this->installment=='yes'){
            ?>
            <div class="form-row form-row-wide">
                <label for="<?=$this->id?>-installment">Taksit Sayısı</label>
                <select class="input-text" id="<?=$this->id?>-installment" name="<?=$this->id?>-installment">
                <?php
                for($i=1;$i<13;$i++){
                    echo '<option>'.$i.'</option>';
                }
                 ?>
                </select>
            </div>
            <?php
            } ?>
           <div class="form-row form-row-wide">
           <label for="<?=$this->id?>-cc-name"><?=__('Card Holder Name',WIG_TEXT_DOMAIN)?> <span class="required">*</span></label>
           <input type="text" class="input-text" placeholder="<?=__('Name',WIG_TEXT_DOMAIN)?>" name="<?=$this->id?>-cc-name" id="<?=$this->id?>-cc-name" />
           </div> 
       <?php }
    }
    public function process_payment($order_id){
       // print_r('asdasdas');return false;
        $order=wc_get_order($order_id);
        global $woocommerce;
       $name=$_POST[$this->id.'-cc-name'];
       $cardNumber=$_POST[$this->id.'-card-number'];
       $cvc=$_POST[$this->id.'-card-cvc'];
       $expiry=$_POST[$this->id.'-card-expiry'];
       $is3d=false;
       $insatallment=isset($_POST[$this->id.'-installment'])?$_POST[$this->id.'-installment']:1;
       $threedMode=$this->get_option('3dMode');
       if($threedMode=="on") $is3d=true;
       if($threedMode=="off") $is3d=false;
       if($threedMode=="auto"){
        $check_ipara = $this->checkInstallment($cardNumber);
        if(!$check_ipara){
            wc_add_notice('Webservise erişimde sorun yaşandı','error');
            return false;
        }
        if ($check_ipara->result == '0') {
            wc_add_notice('Webservise erişimde sorun yaşandı');
            return false;
        }
        if ($check_ipara->supportsInstallment != '1' &&  $insatallment > 1) {
            wc_add_notice('Kartınız taksitli alışverişi desteklemiyor. Lütfen tek çekim olarak deneyiniz','error');
            return false;
        }
        if ($check_ipara->cardThreeDSecureMandatory != '0' || $check_ipara->merchantThreeDSecureMandatory != '0'){
            $is3d=true;
        }else{
            $is3d=false;
        }     
       }
       
       $paymentObject=$this->getPaymentObject($order_id,$name,$cardNumber,$expiry,$cvc,$insatallment,$is3d);
      
       if($is3d){
           try{
          
          $threeDhtml= $paymentObject->payThreeD();
        if(is_wc_endpoint_url('order-pay') && !empty( $_GET['key']) && !empty($_GET['pay_for_order'])){
            echo $threeDhtml;
        }
          return [
            'result' => 'success',
            'messages'=>$threeDhtml,
          ];
           }catch(Exception $e){
               wc_add_notice($e->getMessage(),'error');
               return;
           }
       }
       try {
       $response = $paymentObject->pay();
       $resultStatus=(string)$response['result'];
       $errocode=$response['error_code'];
       $errorText=$response['error_message'];
      
       if($resultStatus=="1"){
        $order->payment_complete();
        $woocommerce->cart->empty_cart();
        return [
            'result' => 'success',
            'redirect' => $this->get_return_url( $order )
        ];
       }else{
            wc_add_notice((string)$errorText,'error');
            return false;
       }
    } catch (Exception $e) {
        wc_add_notice($e->getMessage(),'error');
        return;
    }
    }
    public function validate_fields(){
        $valid=true;
        if(empty($_POST[$this->id.'-cc-name']) || empty($_POST[$this->id.'-card-number']) || empty($_POST[$this->id.'-card-expiry']) || empty($_POST[$this->id.'-card-cvc'])){
            wc_add_notice(__('Payment parameter missing',WIG_TEXT_DOMAIN),'error');
            $valid=false;
        }
        return $valid;
    }
    public function getPaymentObject($order_id,$name,$cardNumber,$expiry,$cvc,$insatallment,$is3d){
        $orderid = $order_id . time();
        $public_key = $this->get_option('publicKey');
        $private_key = $this->get_option('privateKey');
        $expiryArray=explode("/",$expiry);
        $ipara_card=[
            'owner_name'=>$name,
            'number'=>preg_replace("/\s+/", "",$cardNumber),
            'expire_month'=>$expiryArray[0],
            'expire_year'=>$expiryArray[1],
            'cvc'=>$cvc
        ];
        $order=wc_get_order($order_id);
        $ipara_purchaser=[];
        $ipara_purchaser['name'] =$order->get_billing_first_name();
        $ipara_purchaser['surname'] = $order->get_billing_last_name();
        $ipara_purchaser['email'] = $order->get_billing_email();
        $ipara_purchaser['birthdate'] = NULL;
        $ipara_purchaser['gsm_number'] = $order->get_billing_phone();
        $ipara_purchaser['tc_certificate_number'] = NULL;
        $ipara_address=[];
        $ipara_address['name'] = $order->get_billing_first_name();
        $ipara_address['surname'] = $order->get_billing_last_name();
        $ipara_address['address'] = $order->get_billing_address_1();
        $ipara_address['zipcode'] = null;
        $ipara_address['city_code'] = 34;
        $ipara_address['city_text'] = $order->get_billing_city();
        $ipara_address['country_code'] = "TR";
        $ipara_address['country_text'] = "Türkiye";
        $ipara_address['phone_number'] =$order->get_billing_phone();
        $ipara_address['tax_number'] = NULL;
        $ipara_address['tax_office'] = NULL;
        $ipara_address['tc_certificate_number'] = NULL;
        $ipara_address['company_name'] = NULL;

        $items=$order->get_items();
        $ipara_products=[];
        $extra_id = 0;
        foreach ($items as $item_id => $line_item){
                $productId = $line_item['product_id'];
                $variation_id=$line_item['variation_id'];
                $variation=null;
                if($variation_id>0){
                    $variation=new WC_Product_Variation($variation_id);
                    
                }
                $product=wc_get_product($productId);
                $productName=$product->get_title();
                $lineDataDetails=$line_item->get_data();
                if($variation!=null){
                    $productName.=' - '. implode(" / ", $variation->get_variation_attributes());
                }
            $price=$variation!=null?$variation->get_price():$product->get_price();
            $ipara_products[$extra_id]['title'] =$productName;
            $ipara_products[$extra_id]['code'] = $variation_id>0?$variation_id:$productId;   
            $ipara_products[$extra_id]['quantity'] = $lineDataDetails['quantity']; 
            $ipara_products[$extra_id]['price'] =$lineDataDetails['quantity'] * ((int) $price);
            $extra_id++;
        }
        if($this->get_option('shipping')=="yes"){
            $ipara_products[$extra_id]['title'] =__('Shipping Charge',WIG_TEXT_DOMAIN);
            $ipara_products[$extra_id]['code'] = "1".$extra_id;   
            $ipara_products[$extra_id]['quantity'] = 1; 
            $ipara_products[$extra_id]['price'] =$order->get_shipping_total();
            $extra_id++;
        }
        if($this->get_option('tax')=="yes"){
            $ipara_products[$extra_id]['title'] =__('Tax',WIG_TEXT_DOMAIN);
            $ipara_products[$extra_id]['code'] = "1".$extra_id;   
            $ipara_products[$extra_id]['quantity'] = 1; 
            $ipara_products[$extra_id]['price'] =$order->get_total_tax();
            $extra_id++;
        }
        $obj = new iParaPaymentMain();
        $obj->public_key = $public_key;
        $obj->private_key = $private_key;
        $obj->mode = $this->get_option('testMode') == "yes" ? "T" : "P";
        $obj->order_id = $orderid;
        $obj->installment =$insatallment;
        $obj->amount = $order->get_total();
        $obj->vendor_id = 4;
        $obj->echo = "echo message";
        $obj->products = $ipara_products;
        $obj->shipping_address = $ipara_address;
        $obj->invoice_address = $ipara_address;
        $obj->card = $ipara_card;
        $obj->purchaser = $ipara_purchaser;
        if($is3d){
            $orderReturnUrl=$this->get_return_url($order);
            $successString='paymentdone';
            $failedString='paymentfailed';
            $obj->success_url =add_query_arg('web_status',WIG_Common::simple_crypt($order_id.'-'.$successString),$orderReturnUrl);
            $obj->failure_url =add_query_arg('web_status',WIG_Common::simple_crypt($order_id.'-'.$failedString),$orderReturnUrl);;
        }
        return $obj;
    }
    public function checkInstallment($ccNo){
        $publicKey=$this->get_option('publicKey');
        $privateKey=$this->get_option('privateKey');
        $binNumber = substr(preg_replace("/\s+/", "",$ccNo), 0, 6);
        $transactionDate = date("Y-m-d H:i:s");
        $token = $publicKey . ":" . base64_encode(sha1($privateKey . $binNumber . $transactionDate, true));
        $data = ["binNumber" => $binNumber];
        $data_string = json_encode($data);
        $ch = curl_init('https://api.ipara.com/rest/payment/bin/lookup');
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json',
            'Content-Length:' . strlen($data_string),
            'token:' . $token,
            'transactionDate:' . $transactionDate,
            'version:' . '1.0',
        ));
        $response = curl_exec($ch);
        
       if (curl_error($ch)) {
           return false;
       }
       return json_decode($response);
    }
    
}