<?php
/*
Plugin Name: WooCommerce IPARA Gateway
Plugin URI: https://dev.ipara.com.tr/
Description: Allows you to use IPARA payment gateway with the WooCommerce plugin.
Version: 1.0.1
Requires at least: 4.5
Tested up to: 5.6
WC requires at least: 3.5
WC tested up to: 4.9.2
Author: Shirso
Author URI: mailto:shirso@gmail.com
Text Domain: wc_ipara
Domain Path: /languages/
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
if (!defined('WIG_PLUGIN_DIR'))
    define( 'WIG_PLUGIN_DIR', dirname(__FILE__) );
if (!defined('WIG_PLUGIN_ROOT_PHP'))
    define( 'WIG_PLUGIN_ROOT_PHP', dirname(__FILE__).'/'.basename(__FILE__)  );
if(!defined('WIG_PLUGIN_ABSOLUTE_PATH'))
    define('WIG_PLUGIN_ABSOLUTE_PATH',plugin_dir_url(__FILE__));
if (!defined('WIG_PLUGIN_ADMIN_DIR'))
    define( 'WIG_PLUGIN_ADMIN_DIR', dirname(__FILE__) . '/admin' );
if (!defined('WIG_TEXT_DOMAIN'))
    define( 'WIG_TEXT_DOMAIN', 'wc_ipara' );

if( !class_exists('WC_Ipara_Payment') ) {
    class WC_Ipara_Payment{
        public function __construct(){
            if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
                add_action('plugin_loaded',[&$this,'load_main_gateway_class'],0);
                add_filter('plugin_action_links_'.plugin_basename(__FILE__), [&$this,'add_setting_link']);
                add_action( 'template_redirect', [&$this,'update_order_status']);
            }else{
                add_action('admin_notices', [&$this, 'add_notice_for_woocommerce']);
            }
            add_action( 'init', array(&$this,'load_language_file') );
        }
        public function load_language_file(){
            load_plugin_textdomain(WIG_TEXT_DOMAIN, FALSE, dirname(plugin_basename(__FILE__)).'/languages/');
        }
        public function load_main_gateway_class(){
            if (!class_exists('WC_Payment_Gateway')) return;
            require_once( WIG_PLUGIN_DIR.'/includes/ipara-common.php' );
            require_once( WIG_PLUGIN_DIR.'/includes/wc-ipara-gateway.php' );
            add_filter('woocommerce_payment_gateways',[&$this,'add_ipara_gateway']);
        }
        public static function add_ipara_gateway($methods){
            $methods[] = 'Woo_Ipara_Gateway';
            return $methods;
        }
        public static function add_setting_link($links){
            
            array_unshift($links, '<a href="' .
                admin_url( 'admin.php?page=wc-settings&tab=checkout&section=wc_ipara' ) .
                '">' . __('Settings',WIG_TEXT_DOMAIN) . '</a>');
            return $links;
        }
        public static function add_notice_for_woocommerce(){
            ?>
            <div class="notice notice-error">
                <p><strong><?php _e( 'IPARA Payment Gateway requires the WooCommerce plugin to be installed and active.', WIP_TEXT_DOMAIN ); ?></strong></p>

            </div>
            <?php
        }
        public function update_order_status(){
            if( !is_wc_endpoint_url( 'order-received' ) || empty( $_GET['key'] ) || empty($_GET['web_status']) ) {
                return;
            }

            $order_id = wc_get_order_id_by_order_key( $_GET['key'] );
            $order = wc_get_order( $order_id );
            $webStatus=$_GET['web_status'];
            $webStatus=WIG_Common::simple_crypt($webStatus,'d');
            $webStatusArray=explode('-',$webStatus);
            $statusString=$webStatusArray[1];
            if($statusString=='paymentdone'){
                $transactionId=get_post_meta($order_id,'ezpayboTransactionId',true);
                $order->payment_complete($transactionId);
            }else{
                $order->set_status('pending',__('IPARA Payment Failed',WIG_TEXT_DOMAIN));
                wp_redirect($order->get_checkout_payment_url());

            }
        }
    }
    new WC_Ipara_Payment();
}